/*
 *
 * Copyright (C) 2002 George Staikos <staikos@kde.org>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public License
 * along with this library; see the file COPYING.LIB.  If not, write to
 * the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */


#include <kconfig.h>
#include <kdebug.h>

#include "cfgdata.h"

ConfigData::ConfigData(KConfig* config) 
           : _cfg (config)
{
    defaults();
}

ConfigData::~ConfigData()
{
}

ConfigData::ConfigData(const ConfigData& cfg)
{
    *this = cfg;
}

ConfigData const &ConfigData::operator=(ConfigData const &other)
{
    if (this != &other) {

	// General
        autoStart = other.autoStart;
        // maxResolution = other.maxResolution;
        videoDesktopEnabled = other.videoDesktopEnabled;
        useListingsURL = other.useListingsURL;
        listingsURL = other.listingsURL;
	channelNumberingStart = other.channelNumberingStart;
        firstTime = other.firstTime;

	// Device
        lastSource = other.lastSource;
        lastEncoding = other.lastEncoding;
        prevSrc = other.prevSrc;
        prevEncoding = other.prevEncoding;
        prevDev = other.prevDev;

	// Picture
        hue = other.hue;
        colour = other.colour;
        brightness = other.brightness;
        whiteness = other.whiteness;
        contrast = other.contrast;
        autoScaleEnabled = other.autoScaleEnabled;
        
	// Sound
	autoMute = other.autoMute;
	volumeIncrement = other.volumeIncrement;
        volumeRestoreDelay = other.volumeRestoreDelay;

	// Video
        fixAR = other.fixAR;
        ARmode = other.ARmode;

	// Channel
        showSelectedOnly = other.showSelectedOnly;
	channelFile = other.channelFile;
        lastChannel = other.lastChannel;
        
        kdDebug() << "ConfigData::operator = : called..." << endl;
        
	_cfg = other._cfg;
    } // if
    return(*this);
} // operator=


void ConfigData::defaults()
{
    defaultsGeneral();
    defaultsDevice();
    defaultsPicture();
    defaultsChannels();
    defaultsPlugins();
    defaultsSound();
    defaultsVideo();
    defaultsChannels();
} // defaults

void ConfigData::defaultsGeneral()
{
    autoStart = true;
    // maxResolution = false;
    videoDesktopEnabled = true;
    useListingsURL = true;
    listingsURL = QString( "http://www.ananova.com/tv/" );
    channelNumberingStart = 2;
    firstTime = true;
    showSelectedOnly = true;
} // defaultsGeneral

void ConfigData::defaultsDevice()
{
    lastSource   = QString::null;
    lastEncoding = QString::null;
    prevSrc      = QString::null;
    prevEncoding = QString::null;
    return;
} // defaultsDevice


void ConfigData::defaultsPicture()
{
    brightness = hue = colour = contrast = whiteness = 32768;
    autoScaleEnabled = true;
} // defaultsPicture


void ConfigData::defaultsChannels()
{
    // nothing to do...
} // defaultsChannels

void ConfigData::defaultsSound()
{
    autoMute = false;
    volumeIncrement = 4;
    volumeRestoreDelay = 30;
} // defaultsSound

void ConfigData::defaultsVideo()
{
    fixAR = false;
    ARmode = AR_HEIGHT_TO_WIDTH;
} // defaultsVideo

void ConfigData::defaultsPlugins()
{
    // nothing to do... (yet?)
} // defaultsPlugins

int ConfigData::load ()
{
    if (!_cfg)
        return -1;
        
    _cfg->setGroup("General");
    firstTime = _cfg->readBoolEntry("First Time", true);
    autoStart = _cfg->readBoolEntry("Auto Start", true);
    autoMute = _cfg->readBoolEntry("Auto Mute", false);
    // maxResolution = _cfg->readBoolEntry("Capture at Max Res", false);
    videoDesktopEnabled = _cfg->readBoolEntry("Enable Video Desktop", true);
    autoScaleEnabled = _cfg->readBoolEntry("Auto Scale Brightness", true);
    showSelectedOnly = _cfg->readBoolEntry("Show Only Selected Channels", true);
    prevDev = _cfg->readEntry("Previous Device", QString::null);
    prevSrc = _cfg->readEntry("Previous Source", QString::null);
    prevEncoding = _cfg->readEntry("Previous Encoding", QString::null);
    fixAR = _cfg->readBoolEntry("Fix Aspect Ratio", false);
    ARmode = _cfg->readNumEntry("Aspect Ratio Mode", AR_HEIGHT_TO_WIDTH);
    useListingsURL = _cfg->readBoolEntry("Use Listings URL", true);
    listingsURL = _cfg->readEntry("Listings URL", QString::null);
    channelNumberingStart = _cfg->readNumEntry ("Channel Numbering Start", 2);
    volumeRestoreDelay = _cfg->readNumEntry ("Volume Restore Delay", 30);
    volumeIncrement = _cfg->readNumEntry ("Volume Increment", 4);
    
    _cfg->setGroup("Volume");    
    volumeMuted = _cfg->readBoolEntry("Volume Muted", false);
    volumeLeft = _cfg->readNumEntry("Volume Left", 0);     
    volumeRight = _cfg->readNumEntry("Volume Right", 0);

    // Make sure the volume has sane values
    if (volumeLeft > 100)
      volumeLeft = 50;

    if (volumeRight > 100)
      volumeRight = 50;

    return 0;
}


int ConfigData::loadTuning( const QString &dev, const QString &src )
{
    if (!_cfg)
      return -1;

    kdDebug() << "Loading Picture Settings for device: " << dev << endl;
    kdDebug() << "  for source: " << src << endl;
    
    _cfg->setGroup(dev);
    
    if (!_cfg->hasKey(src+" Whiteness")) 
      return -1;

    lastSource   = _cfg->readEntry("Last Source", "");
    lastChannel  = _cfg->readNumEntry("Previous Channel", 2);
    channelFile  = _cfg->readPathEntry("Channel File", QString::null);
    lastEncoding = _cfg->readEntry(src + " Last Encoding", "");
    whiteness    = _cfg->readNumEntry(src + " Whiteness", 0);
    colour       = _cfg->readNumEntry(src + " Colour", 0);
    contrast     = _cfg->readNumEntry(src + " Contrast", 0);
    hue          = _cfg->readNumEntry(src + " Hue", 0);
    brightness   = _cfg->readNumEntry(src + " Brightness", 0);

    kdDebug() << "brightness = " << brightness << " colour = " << colour
              << " hue = " << hue << " contrast = " << contrast << endl;
    return 0;
}


int ConfigData::saveTuning( const QString &dev, const QString &src )
{
    if (!_cfg)
        return -1;

    kdDebug() << "Saving Picture settings for device: " << dev << endl;
    kdDebug() << "  for source: " << src << endl;
    kdDebug() << "brightness = " << brightness << " colour = " << colour << " hue = " << hue << " contrast = " << contrast << endl;

    _cfg->setGroup(dev);
    _cfg->writeEntry("Last Source", lastSource);
    _cfg->writeEntry("Previous Channel", lastChannel);
    _cfg->writePathEntry("Channel File", channelFile);
    _cfg->writeEntry(src + " Last Encoding", lastEncoding);
    _cfg->writeEntry(src + " Whiteness", whiteness);
    _cfg->writeEntry(src + " Colour", colour);
    _cfg->writeEntry(src + " Contrast", contrast);
    _cfg->writeEntry(src + " Hue", hue);
    _cfg->writeEntry(src + " Brightness", brightness);

    return 0;
}


int ConfigData::save() {
    
    if (!_cfg)
       return -1;

    kdDebug() << "save()" << endl;

    _cfg->setGroup("General");
    _cfg->writeEntry("First Time", false);
    _cfg->writeEntry("Auto Start", autoStart);
    _cfg->writeEntry("Auto Mute", autoMute);
    // _cfg->writeEntry("Capture at Max Res", maxResolution);
    _cfg->writeEntry("Enable Video Desktop", videoDesktopEnabled);
    _cfg->writeEntry("Auto Scale Enabled", autoScaleEnabled);
    _cfg->writeEntry("Show Only Selected Channels", showSelectedOnly);
    _cfg->writeEntry("Previous Device", prevDev);
    _cfg->writeEntry("Previous Source", prevSrc);
    _cfg->writeEntry("Previous Encoding", prevEncoding);
    _cfg->writeEntry("Fix Aspect Ratio", fixAR);
    _cfg->writeEntry("Aspect Ratio Mode", ARmode);
    _cfg->writeEntry("Use Listings URL", useListingsURL);
    _cfg->writeEntry("Listings URL", listingsURL);
    _cfg->writeEntry("Channel Numbering Start", channelNumberingStart);
    _cfg->writeEntry("Volume Restore Delay", volumeRestoreDelay);
    _cfg->writeEntry("Volume Increment", volumeIncrement);
 
    _cfg->setGroup("Volume");
    _cfg->writeEntry("Volume Muted", volumeMuted);
    _cfg->writeEntry("Volume Left", volumeLeft);
    _cfg->writeEntry("Volume Right", volumeRight);
    
    return 0;
}

const QString ConfigData::lastSourceFor(const QString& dev) const {
    if (_cfg) 
      return QString::null;

    _cfg->setGroup(dev);
    return _cfg->readEntry("Last Source", QString::null);
}

const QString ConfigData::lastEncodingFor(const QString& dev) const {
    if (_cfg) 
      return QString::null;

    _cfg->setGroup(dev);
    return _cfg->readEntry("Last Encoding", QString::null);
}
