/***************************************************************************
                           channelscanner.cpp
                           ------------------
    begin                : Sat Oct 25 2003
    copyright            : (C) 2003 by Dirk Ziegelmeier
    email                : dziegel@gmx.de
    rcs data             : $Id: channelscanner.cpp,v 1.9 2003/10/26 15:33:31 dziegel Exp $
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <unistd.h>
#include <sys/ioctl.h>
#include <linux/videodev.h>
#include <unistd.h>
#include <fcntl.h>

#include <kdebug.h>
#include <klocale.h>
#include <kled.h>
#include <knuminput.h>
#include <kmessagebox.h>

#include <qcombobox.h>
#include <qapplication.h>
#include <qprogressbar.h>
#include <qlistview.h>
#include <qlabel.h>
#include <qradiobutton.h>
#include <qpushbutton.h>
#include <qstringlist.h>

#include "qtvision.h"
#include "sourcemanager.h"
#include "qvchannelplugin.h"
#include "channel.h"
#include "channelstore.h"
#include "channelimporter.h"
#include "channelscanner.h"
#include "vbidecoder.h"

class ScannerFrequencyFactory
{
public:

    ScannerFrequencyFactory() {};
    virtual ~ScannerFrequencyFactory() {};

    // returns false if no more frequencies
    virtual bool getFrequency(QString& defaultName, ulong& freq, int& percent) = 0;
    virtual bool needsFineTuning() = 0;
    virtual bool forceAdding() = 0;
};

class ScannerFrequencyFactoryPredefined : public ScannerFrequencyFactory
{
private:
    ChannelStore* _store;
    uint          _pos;
    uint          _capacity;
    bool          _forceAdding;

public:
    // Class assumes to be owner of supplied store
    ScannerFrequencyFactoryPredefined(ChannelStore* store, bool force)
	: _store(store),
	  _pos(0),
	  _capacity(store->count()),
	  _forceAdding(force)
    {
    }

    virtual ~ScannerFrequencyFactoryPredefined()
    {
	delete _store;
    }

    virtual bool getFrequency(QString& defaultName, ulong& freq, int& percent)
    {
	if (_pos >= _capacity) {
	    return false;
	}
	Channel* c   = _store->channelAt(_pos);
	freq         = c->freq();
	defaultName  = c->name();
	percent      = (_pos * 100) / _capacity;
	_pos        += 1;
	return true;
    }

    virtual bool needsFineTuning()
    {
	return false;
    }

    virtual bool forceAdding()
    {
	return _forceAdding;
    }
};

class ScannerFrequencyFactoryFreqList : public ScannerFrequencyFactory
{
private:
    ulong _min, _max, _step;
    ulong _current;

public:
    ScannerFrequencyFactoryFreqList(ulong min, ulong max, ulong step)
	: _min(min),
	  _max(max),
	  _step(step),
	  _current(min)
    {
    }

    virtual ~ScannerFrequencyFactoryFreqList() {}

    virtual bool getFrequency(QString& defaultName, ulong& freq, int& percent)
    {
	if (_current > _max) {
	    return false;
	}

	freq         = _current;
	defaultName  = QString::null;
	percent      = ((_current - _min) * 100) / (_max - _min);
	_current    += _step;
	return true;
    }

    virtual bool needsFineTuning()
    {
	return true;
    }

    virtual bool forceAdding()
    {
	return false;
    }
};

ChannelScanner::ChannelScanner( QtVision* qtv, QWidget* parent, const char* name )
    : ChWizard( parent, name, true ),
      _vbiDecoder(0L),
      _store(0L),
      _qtv(qtv),
      _videoDev(qtv->device())
{
    connect(finishButton(), SIGNAL( clicked() ),
	    this, SLOT( finished() ));
    connect(this, SIGNAL( selected(const QString&) ),
	    this, SLOT( pageChanged() ));
    connect(_scanPredefined, SIGNAL( toggled(bool) ),
	    this, SLOT( updatePageSelection() ));
    connect(_scanFrequencies, SIGNAL( toggled(bool) ),
	    this, SLOT( updatePageSelection() ));
    connect(_vbiDevice, SIGNAL( activated(int) ),
	    this, SLOT( checkSignalStrengthReadback() ));
    connect(cancelButton(), SIGNAL( clicked() ),
	    this, SLOT( cancelClicked() ));
    connect(_source, SIGNAL( activated(int) ),
	    this, SLOT( updateSource() ));
    connect(_encoding, SIGNAL( activated(int) ),
	    this, SLOT( updateEncoding() ));

    _source->insertStringList(_qtv->sourceManager()->sourcesFor(_videoDev->device()));
    QString st = _videoDev->source();
    for (int i = 0; i < _source->count(); i++) {
	if (_source->text(i) == st) {
	    _source->setCurrentItem(i);
	    break;
	}
    }

    _encoding->insertStringList(_qtv->sourceManager()->encodingsFor(_videoDev->device()));
    QString et = _videoDev->encoding();
    for (int i = 0; i < _encoding->count(); i++) {
	if (_encoding->text(i) == et) {
	    _encoding->setCurrentItem(i);
	    break;
	}
    }

    _channelImporter = new ChannelImporter();
    _frequencyTable->insertStringList(_channelImporter->lists());

    QString dev("/dev/vbi%1");
    if (!access("/dev/vbi", R_OK)) {
	_vbiDevice->insertItem("/dev/vbi");
    }
    for (int i = 0; i <= 9; i++) {
	if (!access(dev.arg(i).local8Bit(), R_OK)) {
	    _vbiDevice->insertItem(dev.arg(i));
	}
    }
}

ChannelScanner::~ChannelScanner()
{
    delete _channelImporter;
    if (_store) delete _store;
}

void ChannelScanner::updateSource()
{
    _videoDev->setSource(_source->currentText());
}

void ChannelScanner::updateEncoding()
{
    _videoDev->setEncoding(_encoding->currentText());
}

void ChannelScanner::cancelClicked()
{
    _cancel = true;
}

void ChannelScanner::updatePageSelection()
{
    if (_scanPredefined->isChecked()) {
	setAppropriate(((QWizard*)this)->page(preparePredefined), true);
	setAppropriate(((QWizard*)this)->page(prepareFreqRange),  false);
    } else {
	setAppropriate(((QWizard*)this)->page(preparePredefined), false);
	setAppropriate(((QWizard*)this)->page(prepareFreqRange),  true);
    }
}

void ChannelScanner::pageChanged()
{
    helpButton()->setEnabled(false);

    switch (indexOf(currentPage())) {
    case Setup:
	updatePageSelection();
	checkSignalStrengthReadback();
	break;
    case Scanning:
	backButton()->setEnabled(false);
	nextButton()->setEnabled(false);

	if (scan()) {
	    showPage( ((QWizard*)this)->page(Finished) );
	    backButton()->setEnabled(false);
	    nextButton()->setEnabled(true);
	} else {
	    showPage( ((QWizard*)this)->page(Setup) );
	    backButton()->setEnabled(true);
	    nextButton()->setEnabled(true);
	}
	break;
    case Finished:
	finishButton()->setEnabled(true);
	setupFinishedPage();
	break;
    default:
	break;
    }
}

void ChannelScanner::checkSignalStrengthReadback()
{
    int vbiFd = ::open(_vbiDevice->currentText().local8Bit(), O_RDWR);
    ::close(vbiFd);
    if (vbiFd == -1) {
	_vbiAccess->setState(KLed::Off);
    } else {
	_vbiAccess->setState(KLed::On);
    }

    if (_videoDev->signal() == -1) {
	_signalReadback->setState(KLed::Off);
    } else {
	_signalReadback->setState(KLed::On);
    }

    if ( (_videoDev->signal() != -1) ||
	 (vbiFd != -1)                  ) {
	_okToScan->setState(KLed::On);
	nextButton()->setEnabled(true);
    } else {
	_okToScan->setState(KLed::Off);
	nextButton()->setEnabled(false);
    }
}

bool ChannelScanner::scan()
{
    if (_scanPredefined->isChecked()) {
	ChannelStore* fStore = new ChannelStore(_qtv, this, "FactoryStore");
	if (!_channelImporter->import(fStore, _frequencyTable->currentText())) {
	    KMessageBox::error(0L,
			       i18n("Error importing frequency list file. Check your installation!"),
			       i18n("Error reading frequency file"));
	    delete fStore;
	    return false;
	}
	_freqFactory = new ScannerFrequencyFactoryPredefined(fStore, _disableUnused->isChecked());
    } else {
	_freqFactory = new ScannerFrequencyFactoryFreqList((ulong)(_minFreq->value()*1000),
							   (ulong)(_maxFreq->value()*1000),
							   (ulong)(_freqIncrement->value()*1000));
    }

    _vbiDecoder = new VbiDecoder();
    if (_vbiDecoder->init(_vbiDevice->currentText())) {
	_vbiFd = _vbiDecoder->vbiHandle();
    } else {
	_vbiFd = ::open(_vbiDevice->currentText().local8Bit(), O_RDWR);
    }
    _store  = new ChannelStore(_qtv, this, "ResultStore");
    _cancel = false;
    int prevChannel = _qtv->channelNumber();

    doScan();

    _qtv->setChannel(prevChannel);
    delete _freqFactory;
    delete _vbiDecoder;
    ::close(_vbiFd);
    return true;
}

void ChannelScanner::doScan()
{
    while (!_cancel) {
	QString name;
	ulong freq;
	int percent;

	if (!_freqFactory->getFrequency(name, freq, percent)) {
	    return;
	}

	if (name.isEmpty()) {
	    _textualScanFeedback->setText(QString("Scanning... (%1 kHz)").arg(freq));
	} else {
	    _textualScanFeedback->setText(QString("Scanning... (%1)").arg(name));
	}

	_progress->setProgress(percent, 100);
	_videoDev->setFrequency(freq);

	if (isStation()) {
	    if (_freqFactory->needsFineTuning()) {
		freq = fineTune(freq);
	    }

	    _textualScanFeedback->setText(i18n("Station found. Waiting for station name..."));
	    qApp->processEvents();
	    if (_cancel) return;
	    QString vbiName = _vbiDecoder->stationName();
	    if (!vbiName.isEmpty()) {
		name = vbiName;
	    }

	    if (name.isEmpty()) {
		name  = QString("%1 kHz").arg(freq);
	    }

	    kdDebug() << "ChannelScanner: Station found: " << name << ", frequency: " << freq << " kHz" << endl;
	    Channel* c = _store->addChannel(name, freq, _encoding->currentText(), _source->currentText());
	    c->setEnabled(true);
	} else {
	    if (_freqFactory->forceAdding()) {
		Channel* c = _store->addChannel(name, freq, _encoding->currentText(), _source->currentText());
		c->setEnabled(false);
	    }
	}
    }
}

bool ChannelScanner::isStation()
{
    qApp->processEvents();
    if (_cancel) return false;

    // hmmm... what delay works for all? Is there a better way?
    usleep(500000);

    if (_vbiFd != -1) {
	struct video_tuner vt;
	int rc = ioctl(_vbiFd, VIDIOCGTUNER, &vt);
	if (rc < 0) {
	    kdDebug() << "ChannelScanner: IOCTL VIDIOCGTUNER error: " << rc << endl;
	    return false;
	}
	return (vt.signal != 0);
    }
    return (_videoDev->signal() != 0);
}

ulong ChannelScanner::fineTune(ulong freq)
{
    ulong lowBoundary, highBoundary, f;

    f = freq;
    do {
	if (_cancel) return 0;
	lowBoundary = f;
	f -= 50;
	_videoDev->setFrequency(f);
	_textualScanFeedback->setText(i18n("Station found. Finetuning...") + QString(" (%1 kHz)").arg(f));
    } while ( isStation() );

    f = freq;
    do {
	if (_cancel) return 0;
	highBoundary = f;
	f += 50;
	_videoDev->setFrequency(f);
	_textualScanFeedback->setText(i18n("Station found. Finetuning...") + QString(" (%1 kHz)").arg(f));
    } while ( isStation() );

    f = (ulong)(((double)(highBoundary - lowBoundary)) * 0.75) + lowBoundary;
    _videoDev->setFrequency(f);
    return f;
}

void ChannelScanner::setupFinishedPage()
{
    _store->renumber();
    _channelList->clear();
    for (uint i=0; i< _store->count(); i++) {
	new QListViewItem(_channelList, QString("%1").arg(i), _store->channelAt(i)->name());
    }
}

void ChannelScanner::finished()
{
    ChannelStore* qtvStore = _qtv->channels();

    if (_replaceChannels->isChecked()) {
	qtvStore->clear();
    }
    qtvStore->addChannels(*_store);
    _qtv->setChannel(1);
}

#include "channelscanner.moc"
