/*
 *
 * Copyright (C) 2002 George Staikos <staikos@kde.org>
 *               2003 Dirk Ziegelmeier <dziegel@gmx.de>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public License
 * along with this library; see the file COPYING.LIB.  If not, write to
 * the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#include <qfile.h>

#include <kdebug.h>
#include <klibloader.h>
#include <kservice.h>
#include <kservicetype.h>
#include <kconfig.h>
#include <assert.h>

#include "pluginfactory.h"
#include "qtvision.h"

#include "qvpluginbase.h"
#include "qvsrcplugin.h"
#include "qvmixerplugin.h"
#include "qvosdplugin.h"
#include "qvmiscplugin.h"
#include "qvchannelplugin.h"


int PluginFactory::_upid = 0;


PluginFactory::PluginFactory(QtVision *qtv)
    : _qtv(qtv)
{
    _videoPlugins.setAutoDelete(true);
    _channelPlugins.setAutoDelete(true);
    _mixerPlugins.setAutoDelete(true);
    _miscPlugins.setAutoDelete(true);
    _osdPlugins.setAutoDelete(true);
}

PluginFactory::~PluginFactory()
{
}

void PluginFactory::scanForPlugins(KConfig* cfg)
{
    cfg->setGroup("Video Plugins");
    _videoPlugins.clear();
    KService::List vplugs = KServiceType::offers("QtVision Video Source");
    doScan(cfg, vplugs, _videoPlugins, PluginDesc::VIDEO);
    
    cfg->setGroup("Channel Plugins");
    _channelPlugins.clear();
    KService::List cplugs = KServiceType::offers("QtVision Channel Format");
    doScan(cfg, cplugs, _channelPlugins, PluginDesc::CHANNEL);
    
    cfg->setGroup("Mixer Plugins");
    _mixerPlugins.clear();
    KService::List mplugs = KServiceType::offers("QtVision Audio Mixer");
    doScan(cfg, mplugs, _mixerPlugins, PluginDesc::MIXER);
    
    cfg->setGroup("Misc Plugins");
    _osdPlugins.clear();
    KService::List oplugs = KServiceType::offers("QtVision OSD");
    doScan(cfg, oplugs, _osdPlugins, PluginDesc::OSD);
    _miscPlugins.clear();
    KService::List miplugs = KServiceType::offers("QtVision Misc");
    doScan(cfg, miplugs, _miscPlugins, PluginDesc::MISC);
}

QVMixerPlugin*   PluginFactory::getMixerPlugin(PluginDesc* plugin)
{
    if (!plugin) return 0L;

    assert(plugin->type == PluginDesc::MIXER);
    return static_cast<QVMixerPlugin*>(getPluginRefCounted(plugin, false, 0L));
}

QVOSDPlugin*     PluginFactory::getOSDPlugin(PluginDesc* plugin, QWidget* o)
{
    if (!plugin) return 0L;

    assert(plugin->type == PluginDesc::OSD);
    return static_cast<QVOSDPlugin*>(getPluginRefCounted(plugin, true, o));
}

QVMiscPlugin*    PluginFactory::getMiscPlugin(PluginDesc* plugin, QWidget* o)
{
    if (!plugin) return 0L;

    assert(plugin->type == PluginDesc::MISC);
    return static_cast<QVMiscPlugin*>(getPluginRefCounted(plugin, true, o));
}

QVChannelPlugin* PluginFactory::getChannelPlugin(PluginDesc* plugin)
{
    if (!plugin) return 0L;

    assert(plugin->type == PluginDesc::CHANNEL);
    // channel plugins are always enabled
    plugin->enabled = true;
    return static_cast<QVChannelPlugin*>(getPluginRefCounted(plugin, false, 0L));
}


QVSourcePlugin*  PluginFactory::getSourcePlugin(PluginDesc* plugin, QWidget* o)
{
    if (!plugin) return 0L;

    assert(plugin->type == PluginDesc::VIDEO);
    return static_cast<QVSourcePlugin*>(getPluginRefCounted(plugin, true, o));
}

void PluginFactory::putPlugin(PluginDesc* plugin)
{
    if (!plugin) return;

    if (plugin->_instance) plugin->_refCount--;
    kdDebug() << "putPlugin: '" << plugin->name << "' refCount: " << plugin->_refCount << endl;
    if ((plugin->_refCount == 0) && plugin->_instance) {
	plugin->_instance->removeGUIElements(_qtv->guiFactory(), _qtv->actionCollection());
	delete plugin->_instance;
	plugin->_instance = 0L;
    }
}


void PluginFactory::doScan(KConfig* cfg,
			   KService::List& plugs,
			   QPtrList<PluginDesc>& list,
                           PluginDesc::PluginType type)
{
    for (KService::List::ConstIterator it = plugs.begin();
	 it != plugs.end();
	 ++it) {
	KService::Ptr service = *it;
	PluginDesc *x = new PluginDesc(this);
	x->id    = _upid++;
	x->name = service->property("Name").toString();
	x->author = service->property("X-QV-Plugin-Author").toString();
	x->comment = service->property("Comment").toString();
	x->icon = service->property("Icon").toString();
	x->lib = service->property("X-QV-Plugin-Library").toString();
	x->factory = service->property("X-QV-Plugin-Factory").toString();
	x->service = service;
	x->type = type;
	x->configurable = service->property("X-QV-Configurable").toBool();
	if (x->factory.isEmpty())
	    x->factory = x->lib;
	if (!x->factory.startsWith("create_"))
	    x->factory = "create_" + x->factory;
	
	if (cfg->hasKey(x->name+"-"+x->author))
	    x->enabled = cfg->readBoolEntry(x->name+"-"+x->author);
	else x->enabled = service->property("X-QV-Default-Enabled").toBool();
	
	list.append(x);
    }
}

QVPluginBase* PluginFactory::getPluginRefCounted(PluginDesc* plugin, bool qWidgetArg, QWidget* o)
{
    if (plugin->_refCount == 0) {
	plugin->_instance = doGetPlugin(plugin, qWidgetArg, o);
    }
    if (plugin->_instance) plugin->_refCount++;
    kdDebug() << "getPlugin: '" << plugin->name << "' refCount: " << plugin->_refCount << endl;
    return plugin->_instance;
}

QVPluginBase* PluginFactory::doGetPlugin(PluginDesc* plugin, bool qWidgetArg, QWidget* o)
{
    KLibLoader*   loader = KLibLoader::self();
    QVPluginBase* module = 0L;


    if (!plugin->enabled) return 0L;
    
    KLibrary *lib = loader->library(QFile::encodeName(QString("qtvision_")+plugin->lib));
    if (lib) {
	void* create = lib->symbol(QFile::encodeName(plugin->factory));
	if (create) {
	    if (qWidgetArg) {
		QVPluginBase* (*func)(QtVision*, QWidget*);
		func = (QVPluginBase* (*)(QtVision*, QWidget*)) create;
		module = func(_qtv, o);
	    } else {
		QVPluginBase* (*func)(QtVision*);
		func = (QVPluginBase* (*)(QtVision*)) create;
		module = func(_qtv);
	    }

	    if (module) {
		module->_description = plugin;
		module->installGUIElements(_qtv->guiFactory(), _qtv->actionCollection());
		return module;
	    }
	}
    } else {
	kdDebug() << "Error loading library qtvision_" << plugin->lib << endl;
    }
    return 0L;
}



PluginDesc::PluginDesc(PluginFactory* factory)
    : id(-1),
      type(UNKNOWN),
      configurable(false),
      enabled(true),
      _refCount(0),
      _instance(0L),
      _factory(factory)
{
}

PluginDesc::~PluginDesc()
{
    if (_instance) delete _instance;
}

int operator>(const PluginDesc& l, const PluginDesc& r)
{
   return (l.id > r.id) ? 1 : 0;
}

int operator<(const PluginDesc& l, const PluginDesc& r)
{
   return (l.id < r.id) ? 1 : 0;
}

int operator==(const PluginDesc& l, const PluginDesc& r)
{
   return (l.id == r.id) ? 1 : 0;
}

int operator!=(const PluginDesc& l, const PluginDesc& r)
{
   return (l.id == r.id) ? 0 : 1;
}

