/*
 *
 * Copyright (C) 2002 George Staikos <staikos@kde.org>
 *               2003 Dirk Ziegelmeier <dziegel@gmx.de>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public License
 * along with this library; see the file COPYING.LIB.  If not, write to
 * the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */


#ifndef _PLUGINFACTORY_H
#define _PLUGINFACTORY_H

#include <qstring.h>
#include <qwidget.h>
#include <qmap.h>
#include <qptrlist.h>
#include <kservice.h>

class QVPluginBase;
class QVSourcePlugin;
class QVMixerPlugin;
class QVOSDPlugin;
class QVMiscPlugin;
class QVChannelPlugin;
class PluginFactory;

/**
 * Internal description of a plugin
 */
class PluginDesc
{
public:
    typedef enum { UNKNOWN, VIDEO, CHANNEL, MIXER, OSD, MISC, IMAGEFILTER } PluginType;
    PluginDesc(PluginFactory* factory);
    virtual ~PluginDesc();

    int id;
    QString       factory;
    QString       name;
    QString       author;
    QString       comment;
    QString       icon;
    QString       lib;
    KService::Ptr service;
    PluginType    type;
    bool          configurable;
    bool          enabled;

    friend int operator<(const PluginDesc& l, const PluginDesc& r);
    friend int operator>(const PluginDesc& l, const PluginDesc& r);
    friend int operator==(const PluginDesc& l, const PluginDesc& r);
    friend int operator!=(const PluginDesc& l, const PluginDesc& r);

    PluginFactory* getFactory() { return _factory; };

 private:
    friend class PluginFactory;
    int _refCount;
    QVPluginBase*  _instance;
    PluginFactory* _factory;
};


class KConfig;

/**
 * The factory where all the plugins come from.
 * Has lists of plugin descriptors.
 */
class PluginFactory
{
public:
    QPtrList<PluginDesc>& videoPlugins()   { return _videoPlugins;   };
    QPtrList<PluginDesc>& channelPlugins() { return _channelPlugins; };
    QPtrList<PluginDesc>& mixerPlugins()   { return _mixerPlugins;   };
    QPtrList<PluginDesc>& miscPlugins()    { return _miscPlugins;    };
    QPtrList<PluginDesc>& osdPlugins()     { return _osdPlugins;     };

    /**
     * Returns pointer to plugin instance. Instance is refcounted.
     * Don't destroy plugin manually, use putPlugin().
     */
    QVMixerPlugin*   getMixerPlugin(PluginDesc* plugin);
    QVChannelPlugin* getChannelPlugin(PluginDesc* plugin);
    QVOSDPlugin*     getOSDPlugin(PluginDesc* plugin, QWidget* o);
    QVMiscPlugin*    getMiscPlugin(PluginDesc* plugin, QWidget* o);
    QVSourcePlugin*  getSourcePlugin(PluginDesc* plugin, QWidget* o);

    /**
     * To be called if the plugin is no longer used.
     * Destroys plugin if there are no more references.
     * Usually called from aPluginInstance->destroy().
     */
    void putPlugin(PluginDesc* plugin);

protected:
    friend class QtVision;
    PluginFactory(QtVision *qtv);
    virtual ~PluginFactory();

    virtual void scanForPlugins(KConfig* cfg);
    void doScan(KConfig* cfg, KService::List& plugs, QPtrList<PluginDesc>& list, PluginDesc::PluginType type);

    QVPluginBase* getPluginRefCounted(PluginDesc* plugin, bool qWidgetArg, QWidget* o);
    QVPluginBase* doGetPlugin(PluginDesc* plugin, bool qWidgetArg, QWidget* o);

private:
    static int _upid;

    QPtrList<PluginDesc> _videoPlugins;
    QPtrList<PluginDesc> _mixerPlugins;
    QPtrList<PluginDesc> _osdPlugins;
    QPtrList<PluginDesc> _channelPlugins;
    QPtrList<PluginDesc> _miscPlugins;

    QtVision *_qtv;
};



#endif

