/*
   Copyright (c) 2002 Malte Starostik <malte@kde.org>

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public
   License as published by the Free Software Foundation; either
   version 2 of the License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; see the file COPYING.  If not, write to
   the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
   Boston, MA 02111-1307, USA.
*/

// $Id: telex.cpp,v 1.18 2003/10/09 08:25:46 dziegel Exp $

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <errno.h>

typedef unsigned char uint8_t;
#define export export_defanged
#include <libzvbi.h>
#undef export

#include <qbitmap.h>
#include <qcursor.h>
#include <qimage.h>
#include <qpainter.h>
#include <qpixmap.h>
#include <qthread.h>
#include <qlayout.h>
#include <qstring.h>
#include <qlabel.h>
#include <qcombobox.h>

#include <kapplication.h>
#include <kmessagebox.h>
#include <kcursor.h>
#include <kdebug.h>
#include <krun.h>
#include <kaction.h>
#include <klocale.h>
#include <kxmlguifactory.h>
#include <kxmlguiclient.h>
#include <kmainwindow.h>
#include <ktoolbar.h>
#include <kconfig.h>

#include <statusmanager.h>
#include <qtvision.h>

#include <unistd.h>

#include "telex.moc"

#ifdef HAVE_XRENDER_TRANSFORMS
#include <X11/extensions/Xrender.h>
extern bool qt_use_xrender;
#endif

namespace Telex
{
    bool Page::operator ==( const Page& rhs ) const
    {
        return m_number == rhs.m_number &&
               ( m_sub == rhs.m_sub || m_sub == -1 || rhs.m_sub == -1 );
    }

    class Decoder : public QThread
    {
    public:
        Decoder( Plugin* parent );
        virtual ~Decoder();

	bool init( const QString& device );

        void stop() { m_terminator.tryAccess( 1 ); }

        operator vbi_decoder* () const { return m_decoder; }
        int pageOffset() const { return m_pageOffset; }
        void setActive( bool active );
    protected:
        virtual void run();

    private:
        void vbiEvent( vbi_event* event );

        static void vbiEvent( vbi_event* event, void* data )
            { reinterpret_cast< Decoder* >( data )->vbiEvent( event ); }

        QSemaphore m_terminator;
        Plugin* m_parent;

        vbi_capture* m_capture;
        vbi_decoder* m_decoder;
        vbi_sliced* m_sliced;

        int m_pageOffset;
        bool m_active;
    };

    class Event : public QCustomEvent
    {
    public:
        Event( const Page& page )
            : QCustomEvent( User ), m_page( page ) {}

        const Page& page() const { return m_page; }

    private:
        Page m_page;
    };

    Decoder::Decoder( Plugin* parent )
        : QThread(),
          m_terminator( 1 ),
          m_parent( parent ),
          m_capture( 0 ),
          m_decoder( 0 ),
          m_sliced( 0 ),
          m_pageOffset( -1 ),
          m_active( false )
    {
    }

    Decoder::~Decoder()
    {
        stop();
        wait();
	if ( m_decoder ) vbi_event_handler_unregister( m_decoder, &Decoder::vbiEvent, this );
        delete[] m_sliced;
        if ( m_decoder ) vbi_decoder_delete( m_decoder );
        if ( m_capture ) vbi_capture_delete( m_capture );
    }

    bool Decoder::init( const QString& device )
    {
        unsigned int services = VBI_SLICED_TELETEXT_B;
        char* cdevice = const_cast< char* >( device.latin1() );
        if (! ( m_capture = vbi_capture_v4l2_new(cdevice, 625, &services, -1, 0, 0) ) )
            m_capture = vbi_capture_v4l_new(cdevice, 625, &services, -1, 0, 0);
        if ( m_capture )
        {
            m_decoder = vbi_decoder_new();
            vbi_raw_decoder* raw = vbi_capture_parameters( m_capture );
            m_sliced = new vbi_sliced[ raw->count[ 0 ] + raw->count[ 1 ] ];
	    return true;
        } else {
	    return false;
	}
    }

    void Decoder::vbiEvent( vbi_event* event )
    {
        if ( event->ev.ttx_page.roll_header ||
             event->ev.ttx_page.header_update ||
             event->ev.ttx_page.clock_update )
            kapp->postEvent( m_parent, new Event(
                Page( vbi_bcd2dec( event->ev.ttx_page.pgno ),
                      vbi_bcd2dec( event->ev.ttx_page.subno ) ) ) );
        m_pageOffset = event->ev.ttx_page.pn_offset;
    }

    void Decoder::run()
    {
        if ( !m_capture ) return;
        vbi_event_handler_register( m_decoder, VBI_EVENT_TTX_PAGE,
                                    &Decoder::vbiEvent, this );
        double ts = 0.0;
        int lines;
        timeval tv = { 1, 0 };
        while ( m_terminator.available() )
        {
            switch ( vbi_capture_read_sliced( m_capture, m_sliced, &lines, &ts, &tv ) )
            {
                case -1:
                    kdDebug() << "VBI Error: " << strerror( errno ) << endl;
                    return;
                case 0: continue;
                case 1: vbi_decode( m_decoder, m_sliced, lines, ts );
            }
            if( ! m_active ) msleep(200);
        }
    }

    void Decoder::setActive( bool active )
    {
        m_active = active;
    }

    bool Link::operator ==( const Link& rhs ) const
    {
        if ( rhs.m_type != m_type ) return false;
        if ( m_type == TTX ) return rhs.m_page == m_page;
        else return rhs.m_url == m_url;
    }

    Plugin::Plugin( QtVision *qtv, QWidget* parent )
        : DCOPObject("TelexIface"), 
	  QVMiscPlugin( qtv, "telex-misc", parent ),
          m_decoder( 0 ),
          m_showAction( 0 ),
          m_transparentAction( 0 ),
          m_pageInput( 0 )
    {
        if ( !parent ) return;

        m_display = new Display( parent, this );

	_cfg->setGroup("Device");
	_cfg_dev = _cfg->readEntry("Device", "/dev/vbi");
	kdDebug() << "Telex: using device: " << _cfg_dev << endl;

        connect( m_display, SIGNAL( navigate( const Link& ) ),
                 SLOT( navigate( const Link& ) ) );
        connect( driver()->statusManager(), SIGNAL( channelChanged() ),
                 SLOT( channelChanged() ) );
        // disable display it will be enabled by the user
        m_display->hide();
    }

    Plugin::~Plugin()
    {
        delete m_display;
        delete m_decoder;
    }

    void Plugin::navigate( int page )
    {
        if ( page > 100 )
            navigate( Page( page ) );
    }

    void Plugin::nextPage()
    {
        navigate( m_page.number() + 1 );
    }

    void Plugin::previousPage()
    {
        navigate( m_page.number() - 1 );
    }

    void Plugin::installGUIElements(KXMLGUIFactory *guiFactory,
            KActionCollection *actionCollection)
    {
        if ( actionCollection )
        {
            m_showAction =
                new KToggleAction( i18n( "Show Teletext" ), "text_center", 0,
                                   actionCollection, "toggle_teletext" );
            m_showAction->setChecked(false);
            connect( m_showAction,
                     SIGNAL( toggled( bool ) ),
                     SLOT( showDisplay( bool ) ) );
            m_transparentAction =
                new KToggleAction( i18n( "Transparent Teletext" ), "view_text",
                                   0, actionCollection,
                                   "toggle_teletext_transparent" );
            m_transparentAction->setChecked( false );
            m_transparentAction->setEnabled( false );
            connect( m_transparentAction,
                     SIGNAL( toggled( bool ) ),
                     m_display,
                     SLOT( setTransparent( bool ) ) );
            if (guiFactory) {
                // FIXME there must be an easier way to get the toolbar
                // get first client
                KXMLGUIClient* client = guiFactory->clients().getFirst();
                KMainWindow* mw = dynamic_cast<KMainWindow*>(client);
                if (mw)
                {
                    QWidget* widget = mw->toolBar("viewToolBar");
                    if (widget)
                    {
                        m_showAction->plug(widget);
                        m_transparentAction->plug( widget );
                    }
                    else
                        kdDebug() << "Telex: No 'viewToolBar' widget" << endl;
                    // plug it to popup menu
                    //QWidget *pop = (QWidget*)guiFactory->container("screen_context_popup", mw);
                    QWidget *pop = (QWidget*)guiFactory->container("screen_context_popup", client);
                    if (pop)
                    {
                        m_showAction->plug(pop);
                        m_transparentAction->plug( pop );
                    }
                    else
                        kdDebug() << "Telex: No 'screen_context_popup' widget" << endl;
                }
            }
        }
    }

    void Plugin::removeGUIElements(KXMLGUIFactory *guiFactory, KActionCollection*)
    {
        if ( guiFactory ) {
	    // FIXME there must be an easier way to get the toolbar
	    // get first client
	    KXMLGUIClient* client = guiFactory->clients().getFirst();
	    KMainWindow* mw = dynamic_cast<KMainWindow*>(client);
	    if (mw) {
		QWidget* widget = mw->toolBar("viewToolBar");
		if (widget) {
		    m_showAction->unplug(widget);
		    m_transparentAction->unplug( widget );
		}
		else
		    kdDebug() << "Telex: No 'viewToolBar' widget" << endl;

		QWidget *pop = (QWidget*)guiFactory->container("screen_context_popup", client);
		if (pop) {
		    m_showAction->unplug(pop);
		    m_transparentAction->unplug( pop );
		}
		else
		    kdDebug() << "Telex: No 'screen_context_popup' widget" << endl;
	    }
        }
    }

    bool Plugin::filterNumberKey( int key )
    {
        if (m_display->isHidden())
            return false;
        if ( key == -1 ) return false;
        if ( m_pageInput >= 100 ) m_pageInput = key;
        else m_pageInput = m_pageInput * 10 + key;
        if ( m_pageInput >= 100 ) navigate( Page( m_pageInput ) );
        return true;
    }

    QWidget* Plugin::configWidget( QWidget* parent, const char* name )
    {
	QFrame *w = new QFrame(parent, name);
	w->setMargin(4);

	QGridLayout *g = new QGridLayout(w, 1, 3);
	
	g->addWidget(new QLabel(i18n("Device:"), w), 0, 0);
	_cb_dev = new QComboBox(w);
	g->addMultiCellWidget(_cb_dev, 0, 0, 1, 2);

	QString dev("/dev/vbi%1");
	QStringList _devlist;

	if (!access("/dev/vbi", R_OK)) {
	    _devlist << "/dev/vbi";
	}

	for (int i = 0; i <= 9; i++) {
	    if (!access(dev.arg(i).local8Bit(), R_OK)) {
		_devlist << dev.arg(i);
	    }
	}

	_cb_dev->insertStringList(_devlist);
	_cb_dev->setCurrentText(_cfg_dev);

	return w;
    }

    void Plugin::saveConfig()
    {
	_cfg->setGroup("Device");
	_cfg->writeEntry("Device", (_cfg_dev = _cb_dev->currentText()));
	_cfg->sync();
    }

    void Plugin::channelChanged()
    {
        if( m_decoder )
            m_display->setPage( m_page );
    }

    void Plugin::showDisplay(bool p_show)
    {
        if( !m_decoder ) {
            m_decoder = new Decoder( this );
	}

	if (!m_decoder->running()) {
            if ( !m_decoder->init(_cfg_dev.local8Bit()) ) {
                KMessageBox::error(0L,
                    i18n("Unable to initialize Telex plugin.\nCheck whether configured /dev/vbiX is accessible"),
                    i18n("Error: connot initialize Telex plugin"));

		// keep UI synced
		m_showAction->setChecked( false );
		m_transparentAction->setEnabled( false );
                return;
            }
            m_decoder->start();
            navigate( Page( 100 ) );
        }
        
        m_decoder->setActive(p_show); 
        
        if (p_show)
            m_display->show();
        else
            m_display->hide();
        m_transparentAction->setEnabled( p_show );
    }

    void Plugin::toggleShow()
    {
        showDisplay(m_display->isHidden());
        m_showAction->setChecked( !m_display->isHidden() );
    }

    void Plugin::toggleTransparent()
    {
        m_display->setTransparent( !m_display->isTransparent() );
        m_transparentAction->setChecked( m_display->isTransparent() );
    }

    void Plugin::navigate( const Link& link )
    {
        switch ( link.type() )
        {
            case Link::TTX:
                if( m_decoder )
                    m_display->setPage( m_page = link.page() );
                break;
            case Link::URL: new KRun( link.url() );
            default: break;
        }
    }

    void Plugin::customEvent( QCustomEvent* e )
    {
        Event* event = static_cast< Event* >( e );
        if ( event->page() == m_page ) m_display->setPage( event->page() );
        else m_display->setHeader( event->page() );
    }

    Display::Display( QWidget* parent, Plugin* plugin )
        : QWidget( parent ),
          m_plugin( plugin ),
          m_transparent( false ),
          m_columns( 0 ),
          m_rows( 0 )
    {
        setBackgroundMode( NoBackground );
        parent->installEventFilter( this );
        parent->setMouseTracking( true );
        setMouseTracking( true );
        resize( parent->size() );
        setAutoMask( true );
    }

    Display::~Display()
    {
        parentWidget()->setMouseTracking( false );
        parent()->removeEventFilter( this );
    }

    void Display::setHeader( const Page& page )
    {
        if ( fetch( page, true ) )
            update( 0, 0, width(), height() / 25 );
    }

    void Display::setPage( const Page& page )
    {
        if ( fetch( page, false ) )
        {
            m_page = page;
            updateMask();
            update();
        }
    }

    bool Display::fetch( const Page& page, bool headerOnly )
    {
        int rows[] = { 25, 1 };
        vbi_page data;
        if ( !vbi_fetch_vt_page( *m_plugin->decoder(),
                                 &data,
                                 vbi_dec2bcd( page.number() ),
                                 page.hasSub() ?
                                    vbi_dec2bcd( page.sub() ) :
                                    VBI_ANY_SUBNO,
                                 VBI_WST_LEVEL_3p5,
                                 rows[ headerOnly ],
                                 !headerOnly ) )
                return false;

        int offset = m_plugin->decoder()->pageOffset();
        if ( offset != -1 )
        {
            QString info = QString::number( m_plugin->page().number() );
            for ( int i = 0; i < 3; ++i )
            {
                data.text[ offset + i ].unicode = info[ i ].unicode();
                if ( m_plugin->page() != m_page )
                    // XXX: I want it red, and VBI_BLUE appears as red for me
                    // while VBI_RED appears as blue. Anyone else?
                    data.text[ offset + i ].foreground = VBI_BLUE;
            }
        }

        QImage image( data.columns * 12, data.rows * 10,
                      32, 0, QImage::LittleEndian );

        vbi_draw_vt_page( &data, VBI_PIXFMT_RGBA32_LE,
                          image.bits(), false, true );

        if ( !headerOnly )
        {
            image.setAlphaBuffer( true );
            Q_UINT32* p =
                reinterpret_cast< Q_UINT32* >( image.scanLine( 10 ) );
            Q_UINT32* end = reinterpret_cast< Q_UINT32* >(
                image.scanLine( image.height() - 1 ) );
            int x = 0, y = 10;
            vbi_opacity opacity;
            for ( ; p < end; ++p )
            {
                opacity = vbi_opacity(
                    data.text[ y / 10 * data.columns + x / 12 ].opacity );
                switch ( opacity )
                {
                    case VBI_TRANSPARENT_SPACE:
                        *p = 0;
                        break;
                    case VBI_OPAQUE:
                        if ( !m_transparent ) break;
                    case VBI_TRANSPARENT_FULL:
                    case VBI_SEMI_TRANSPARENT:
                        if ( *p == qRgba( 0, 0, 0, 0xff ) ) *p = 0;
                        break;
                    default: break;
                }
                if ( ++x >= image.width() )
                {
                    x = 0;
                    ++y;
                }
            }

            m_columns = data.columns;
            m_rows = data.rows;
            vbi_link link;
            m_links.resize( m_rows * m_columns );
            for ( int row = 0; row < m_rows; ++row )
                for ( int col = 0; col < m_columns; ++col )
                {
                    int index = row * m_columns + col;
                    vbi_resolve_link( &data, col, row, &link );
                    switch ( link.type )
                    {
                        case VBI_LINK_PAGE:
                        case VBI_LINK_SUBPAGE:
                            m_links[ index ] =
                                Page( vbi_bcd2dec( link.pgno ),
                                      ( link.subno == VBI_ANY_SUBNO ) ?
                                          -1 : link.subno );
                            break;
                        case VBI_LINK_HTTP:
                        case VBI_LINK_FTP:
                        case VBI_LINK_EMAIL:
                            m_links[ index ] = KURL(
                                reinterpret_cast<const char *>( link.url ) );
                            break;
                        default:
                            m_links[ index ] = Link();
                    }
                }
        }

        QPixmap pixmap;
        pixmap.convertFromImage( image );
        if ( m_pixmap.width() == data.columns * 12 && headerOnly )
        {
            bitBlt( &m_pixmap, 0, 0, &pixmap );
#ifdef HAVE_XRENDER_TRANSFORMS
            if ( !qt_use_xrender )
#endif
            {
                QPixmap scaled;
                int h = pixmap.height() > 10 ? height() : height() / 25;
                scaled.convertFromImage( pixmap.convertToImage()
                    .smoothScale( width(), h ) );
                bitBlt( &m_scaled, 0, 0, &scaled );
            }
        }
        else
        {
            m_pixmap = pixmap;
            updateScale();
        }
        return true;
    }

    void Display::updateScale()
    {
#ifdef HAVE_XRENDER_TRANSFORMS
        if ( qt_use_xrender )
        {
            int h = m_pixmap.height() > 10 ? height() : height() / 25;
            XTransform transform =
            {
                {
                    { 1000 * m_pixmap.width() / width(), 0, 0 },
                    { 0, 1000 * m_pixmap.height() / h, 0 },
                    { 0, 0, 1000 }
                }
            };
            XRenderSetPictureTransform( qt_xdisplay(),
                                        m_pixmap.x11RenderHandle(),
                                        &transform );
            if ( m_pixmap.mask() )
                XRenderSetPictureTransform( qt_xdisplay(),
                                            m_pixmap.mask()->x11RenderHandle(),
                                            &transform );
/*            XRenderSetPictureFilter( qt_xdisplay(),
                                     m_pixmap.x11RenderHandle(),
                                     "good", 0, 0 );*/
        }
        else
#endif
        {
            int h = m_pixmap.height() > 10 ? height() : height() / 25;
            m_scaled.convertFromImage( m_pixmap.convertToImage()
                .smoothScale( width(), h ) );
        }
    }

    void Display::setTransparent( bool transparent )
    {
        m_transparent = transparent;
        updateMask();
        setPage( m_page );
    }

    void Display::updateMask()
    {
        if ( m_pixmap.height() <= 10 )
            setMask( QRect( 0, 0, width(), height() / 25 ) );
        else if ( m_pixmap.mask() )
        {
#ifdef HAVE_XRENDER_TRANSFORMS
            if ( qt_use_xrender )
            {
                QBitmap mask( width(), height() );
                XRenderComposite( qt_xdisplay(), PictOpSrc,
                                  m_pixmap.mask()->x11RenderHandle(), 0,
                                  mask.x11RenderHandle(),
                                  0, 0, 0, 0, 0, 0, width(), height() );
                setMask( mask );
            }
            else
#endif
                setMask( *m_scaled.mask() );
        }
        else clearMask();
    }

    bool Display::eventFilter( QObject*, QEvent* e )
    {
        if ( e->type() == QEvent::Resize )
        {
            resize( static_cast< QResizeEvent* >( e )->size() );
            return false;
        }
        if ( e->type() == QEvent::MouseButtonPress )
            mousePressEvent( static_cast< QMouseEvent* >( e ) );
        else if ( e->type() == QEvent::MouseButtonRelease )
            mouseReleaseEvent( static_cast< QMouseEvent* >( e ) );
        else if ( e->type() == QEvent::MouseMove )
            mouseMoveEvent( static_cast< QMouseEvent* >( e ) );
        else return false;
        return true;
    }

    void Display::resizeEvent( QResizeEvent* e )
    {
        if ( !m_pixmap.isNull() ) updateScale();
        QWidget::resizeEvent( e );
    }

    void Display::paintEvent( QPaintEvent* e )
    {
        if ( !m_pixmap.isNull() )
        {
#ifdef HAVE_XRENDER_TRANSFORMS
            if ( qt_use_xrender )
                XRenderComposite( qt_xdisplay(), PictOpSrc,
                                  m_pixmap.x11RenderHandle(), 0,
                                  x11RenderHandle(),
                                  e->rect().left(), e->rect().top(),
                                  0, 0,
                                  e->rect().left(), e->rect().top(),
                                  e->rect().width(), e->rect().height() );
            else
#endif
                bitBlt( this, e->rect().topLeft(), &m_scaled, e->rect() );
        }
    }

    void Display::mouseMoveEvent( QMouseEvent* e )
    {
        if ( !m_rows || !m_columns ) return;
        int col = m_columns * e->x() / width(),
            row = m_rows * e->y() / height();
        if ( m_links[ row * m_columns + col ] )
            parentWidget()->setCursor( KCursor::handCursor() );
        else parentWidget()->setCursor( KCursor::arrowCursor() );
    }

    void Display::mousePressEvent( QMouseEvent* e )
    {
        if ( !m_rows || !m_columns ) return;
        int col = m_columns * e->x() / width(),
            row = m_rows * e->y() / height();
        emit navigate( m_links[ row * m_columns + col ] );
    }
}

extern "C"
{
    QVMiscPlugin* create_telex( QtVision *qtv, QWidget* parent )
    {
        return new Telex::Plugin( qtv, parent );
    }
}

// vim: ts=4 sw=4 et
