// -*- c++ -*-

/*
 *
 * Copyright (C) 2002 George Staikos <staikos@kde.org>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public License
 * along with this library; see the file COPYING.LIB.  If not, write to
 * the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#ifndef __QTVISION_H
#define __QTVISION_H

#include <qmap.h>
#include <qcolor.h>
#include <qobject.h>
#include <qpixmap.h>
#include <qstring.h>
#include <qguardedptr.h>

#include "cfgdata.h"
#include "channel.h"

#include "qtvisioniface.h"
#include "qvsrcplugin.h"
#include "view.h"

class KConfig;

class ViewManager;
class AudioManager;
class SourceManager;
class OSDManager;
class ChannelStore;
class StatusManager;
class MiscManager;
class PluginFactory;
class KActionCollection;
class KXMLGUIFactory;

/**
 * A reusable TV object.
 */
class QtVision : public QObject, virtual public QtVisionIface
{
    Q_OBJECT

public:
    QtVision( QObject *parent = 0, const char* name = 0);
    virtual ~QtVision();

    /**
     * Returns the current screen widget.
     */
    QtVisionView *screen() { return _view; }

    /**
     * Returns a DCOP reference to the ChannelStore.
     */
    DCOPRef channelStoreIface();

    /**
     * Returns a DCOP reference to the current Channel.
     */
    DCOPRef channelIface();

    /**
     * Returns the name of the current Channel.
     */
    QString channelName() const;

    /**
     * Returns the number of the current Channel.
     */
    int channelNumber() const;

    /**
     * Returns true iff we have a video device.
     */
    bool hasDevice() const { return _vsrc ? true : false; }

    /**
     * Returns the current video device (or 0).
     */
    QVSourcePlugin *device() const { return _vsrc; }

    /**
     * Returns the manager controlling the views.
     */
    ViewManager *views() const { return _viewmng; }

    /**
     * Returns the audio manager.
     */
    AudioManager *audioManager() const { return _am; }

    /**
     * Returns the source manager.
     */
    SourceManager *sourceManager() const { return _srcm; }

    /**
     * Returns the OSD manager.
     */
    OSDManager *osdManager() const { return _osd; }

    /**
     * Returns the miscellaneous plugin manager.
     */
    MiscManager *miscManager() const { return _mm; }

    /**
     * Returns the status plugin manager.
     */
    StatusManager *statusManager() const { return _sm; }

    /**
     * Returns the plugin factory.
     */
    PluginFactory *pluginFactory() const { return _pf; }

    /**
     * Returns current ChannelStore.
     */
    ChannelStore *channels() { return _cs; }

    /**
     * Returns the current configuration.
     */
    ConfigData *config() const { return _cfg; }
    
    /**
     * Returns the KConfig objectg for this instance.
     */
    KConfig *configHandle () const { return _config; }

    /**
     * Returns list of supported channel formats with read support.
     * For use with importChannelFile()
     */
    const QStringList& fileFormatsRead() const;

    /**
     * Returns list of supported channel formats with write support.
     * For use with exportChannelFile()
     */
    const QStringList& fileFormatsWrite() const;

    /**
     * Factory method for creating the screen the picture will be displayed
     * on. This widget returned will depend on the facilities supported by
     * the TV card and X server (eg. there will be special screen widgets
     * for Xv support). Note that this widget is automatically made the
     * screen for this QtVision object.
     */
    virtual QtVisionView *createScreen( QWidget *parent=0, const char *name=0 );

    /**
     * Return a copy of the client's GUI factory, if available, so that plugins
     * or other components can modify the UI dynamically.
     * May return 0L!
     * Note: After the guiFactory is set all the plugins are notified of the (new) gui
     * factory so they usually dont need to call this.
     * This method is only needed for the plugin factory when creating new plugins.
     */
    virtual KXMLGUIFactory *guiFactory() const;

    /**
     * Return a copy of the client's action collection, if available, so that
     * plugins or other components can modify the UI dynamically.
     * May return 0L!
     * Note: After the actionCollection is set all the plugins are notified of the (new) action
     * collection so they usually dont need to call this.
     * This method is only needed for the plugin factory when creating new plugins.
     */
    virtual KActionCollection *actionCollection() const;

    /**
     * To get various values.
     */
    int brightness();
    int colour();
    int hue();
    int contrast();
    int whiteness();
    QColor colourKey();
    Channel *channel() { return _chan; }

    /**
     * FOR CLIENTS ONLY!  Set the guiFactory and actionCollection if available.
     */
    virtual void setGuiFactory(KXMLGUIFactory *guiFactory, KActionCollection *actionCollection);

public slots:
    /**
     * Try to start using the previous device. Returns true iff it succeeds.
     */
    bool tryAutoStart();

    /**
     * Ask the user to select a device.
     */
    virtual void selectDevice();

    /**
     * Set the current video device.
     */
    void setDevice( QVSourcePlugin *dev );

    /**
     * Sets the source to use.
     */
    virtual void setSource( const QString &src );

    /**
     * Sets the tuner mode.
     */
    void setEncoding( const QString& encoding );

    /**
     * Stop the current device and dispose of it.
     */
    virtual void stopDevice();

    /**
     * Sets the current current channel.
     */
    void setChannel( Channel *channel );

    /**
    * Overloaded version of the above method
    */
    void setChannel(int channel);
    
    
    /**
     * Changes the channel to the one preceding the current, wrapping
     * round if we are currently showing the first channel.
     */
    virtual void channelDown();

    /**
     * Changes the channel to the one following the current, wrapping
     * round if we are currently showing the last channel.
     */
    virtual void channelUp();
    
    /**
     * Recalls the previous channel.
     */
    virtual void previousChannel();

    /**
     * Mutes and unmutes the sound.
     */
    virtual void volumeMute();

    /**
     * Increases the volume.
     */
    virtual void volumeUp();

    /**
     * Decreases the volume.
     */
    virtual void volumeDown();

    /**
     * Sets the volume
     */
    virtual void setVolume(int left, int right);
    virtual void setVolume(int vol);

    virtual void start();
    virtual void stop();
    virtual void startCapture();
    virtual void snapshot();

    virtual void startVideo();
    virtual void stopVideo();

    /**
     * Launches the settings dialog.
     */
    virtual void settings();

    /**
     * Launches the channel wizard and returns the dialog return value.
     */
    virtual int launchWizard();

    /**
     * Set various attributes on the current device.
     */
    virtual void setBrightness(int val);
    virtual void setColour(int val);
    virtual void setHue(int val);
    virtual void setContrast(int val);
    virtual void setWhiteness(int val);

    /**
     *  Set video desktop on or off
     */
    virtual void setVideoDesktop(bool on);

    /**
     *  Do migration from kwintv to qtvision
     */
    virtual bool doMigration();

    /**
     *  Imports the old KWinTV channels if they exist.
     */
    virtual bool importLegacyChannels();

    /**
     *  Imports default channel list
     */
    virtual void importDefaultChannels();

    /**
     *  Open a file dialog to allow the user to add the channels from a file
     *  to the channel list.
     */
    virtual void importChannelFile(const QString& fmt=QString::null);

    /**
     *  Open a file dialog to allow the user export channels in selected format
     */
    virtual void exportChannelFile(const QString& fmt=QString::null);

    /**
     * Process the press of a number key (0-9) from keyboard, or lirc.
     */
    virtual void processNumberKeyEvent(int);

    /**
     * Triggers sourcemanager to rescan source plugins.
     * Necessary because current source plugin must be dropped before.
     */
    virtual void rescanSourcePlugins();

    /**
     * Reload channel file
     */
    virtual void reloadChannels();

    /**
     * Save channel file
     */
    virtual void saveChannels();

signals:
    /**
     * Emitted when the volume changes
     */
    void volumeChanged( int left, int right );
    
    /**
     * Emitted whenever the volume is mutted.
     */
    void volumeMuted(bool);

    /**
     * Emitted when the current channel changes.
     */
    void channelChanged( int num );

    /**
     * Emitted when the current channel changes.
     */
    void channelChanged( const QString &name );

    /**
     * Emitted when the current channel changes.
     */
    void channelChanged( Channel *channel );

    /**
     * Emitted when the video device changes.
     */
    void deviceChanged( QVSourcePlugin *dev );

    /**
     * Emitted when the channel text changes during setting channel with numeric keys.
     */
    void channelText(const QString &);

protected:
    void timerEvent (QTimerEvent *ev);
    
private slots:
    void viewResized(int w, int h);
    void viewMoved(int x, int y);
    void aboutToQuit();
    
    /** Mute/Unmute volume... */
    void setMuteVolume (bool);

    void slotKeyPressTimeout();

private:
    void setLastChannel();

    ChannelStore *_cs;

    QGuardedPtr<Channel> _chan;

    OSDManager *_osd;
    AudioManager *_am;
    SourceManager *_srcm;
    StatusManager *_sm;
    MiscManager *_mm;
    QtVisionView *_view;
    PluginFactory *_pf;
    QVSourcePlugin *_vsrc;
    ViewManager *_viewmng;
    ConfigData* _cfg;
    KConfig* _config;
    QPixmap grabPix;
    KXMLGUIFactory *_guiFactory;
    KActionCollection *_actionCollection;

    QStringList _fileFormatsRead;
    QStringList _fileFormatsWrite;

    QTimer *_keypresstimer;
    QString _number;

    int _prevChannel;

    bool _muted;    // We have to keep track since some devices are broken    
    bool _quitting;

    struct {
        int left; // Left volume
        int right; // Right volume
        int changeEventId; // Id 
    } ChannelVolState;

    unsigned long _grabNumber;
};

#endif

