/*
 *
 * Copyright (C) 2002 Rich Moore <rich@kde.org>
 * Copyright (C) 2002 George Staikos <staikos@kde.org>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public License
 * along with this library; see the file COPYING.LIB.  If not, write to
 * the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */


#include <qcheckbox.h>
#include <qcombobox.h>
#include <qhbox.h>
#include <qmessagebox.h>
#include <qprogressbar.h>
#include <qpushbutton.h>
#include <qtabdialog.h>
#include <qslider.h>
#include <qspinbox.h>

#include <kapplication.h>
#include <kdebug.h>
#include <kdialogbase.h>
#include <klocale.h>
#include <kmessagebox.h>

#include "chwizard.h"
#include "channelscanner.h"
#include "channeleditor.h"
#include "devicewidgetimpl.h"
#include "qtvision.h"
#include "qvsrcplugin.h"
#include "pluginfactory.h"
#include "settingsdialog.h"
#include "settingsdialogpage.h"
#include "cfgdata.h"
#include "channelimportdlgimpl.h"

#include "viewmanager.h"
#include "viewmanager.moc"

ViewManager::ViewManager( QtVision *qtvision, const char *name )
    : QObject( qtvision, name ? name : "view_manager" ),
      dev(0), qtv( qtvision ), cfg( qtvision->config() ),
      oldConfig(ConfigData(0L)),
      settingsDialog(0L)
{
    connect( qtv, SIGNAL( deviceChanged( QVSourcePlugin * ) ), this, SLOT( update() ) );
}

ViewManager::~ViewManager()
{

}

void ViewManager::update()
{
    dev = qtv->device();
    cfg = qtv->config();
}

int ViewManager::launchSourceDialog( QWidget *parent )
{
    kdDebug() << "ViewManager::launchSourceDialog()" << endl;

    KDialogBase *dlg = new KDialogBase(parent, "ssdlg", true, i18n("Select Source"),
                                       KDialogBase::Help | KDialogBase::Ok | KDialogBase::Cancel,
                                       KDialogBase::Ok, true);

    QHBox *hbox = dlg->makeHBoxMainWidget();
    SettingsDialogPage* ssdlg = dynamic_cast<SettingsDialogPage*>(new DeviceWidgetImpl(hbox, qtv, cfg, qtv->configHandle()));
    ssdlg->setup();

    int rc = dlg->exec();

    if ( rc == KDialogBase::Accepted ) {
	ssdlg->apply();
    }

    delete dlg;
    dlg = 0;
    return(rc);
}


int ViewManager::launchImportDefault( QWidget *parent )
{
    ChannelImportDlgImpl d(qtv, parent, "ImportDlg", true);
    return d.exec();
}


int ViewManager::launchWizard( QWidget* parent )
{
    if (!qtv->device()) return -1;

    ChannelScanner* cs = new ChannelScanner( qtv, parent, "Channel Wizard" );
    return cs->exec();
}


int ViewManager::launchSettings( QWidget *parent )
{
    oldConfig = *cfg;

    QPtrList<SettingsDialogPage> p;
    for (QPtrListIterator<SettingsDialogPageFactory>fit(_pageFactories);
	 fit.current() != 0;
	 ++fit) {
	QPtrList<SettingsDialogPage>* temp = (*fit)->getPages();
	
	for (QPtrListIterator<SettingsDialogPage>pit(*temp);
	     pit.current() != 0;
	     ++pit) {
	    p.append(*pit);
	}

	delete temp;
    }

    settingsDialog = new SettingsDialog(parent, 0, true, qtv, &p);

    connect(settingsDialog, SIGNAL( optionsModified() ),
	    this, SLOT( setOptions() ));
    connect(settingsDialog, SIGNAL( finished() ),
	    this, SLOT( destroySettings() ));

    settingsDialog->show();
    return(0); // success
}

void ViewManager::destroySettings() {
    if (settingsDialog) {
        settingsDialog->delayedDestruct();
        settingsDialog = 0;
    }
}

void ViewManager::registerPageFactory(SettingsDialogPageFactory* f)
{
    _pageFactories.append(f);
}

void ViewManager::unregisterPageFactory(SettingsDialogPageFactory* f)
{
    _pageFactories.remove(f);
}

void ViewManager::slotHueChanged(int hue)
{
    qtv->setHue(hue);
    return;
} // slotHueChanged

void ViewManager::slotColourChanged(int colour)
{
    qtv->setColour(colour);
    return;
} // slotColourChanged

void ViewManager::slotBrightnessChanged(int brightness)
{
    qtv->setBrightness(brightness);
    return;
} // slotBrightnessChanged

void ViewManager::slotWhitenessChanged(int whiteness)
{
    qtv->setWhiteness(whiteness);
    return;
} // slotWhitenessChanged

void ViewManager::slotContrastChanged(int contrast)
{
    qtv->setContrast(contrast);
    return;
} // slotContrastChanged

void ViewManager::setOptions()
{ 
    if (oldConfig.showSelectedOnly != cfg->showSelectedOnly)
        emit showSelectedChanged(cfg->showSelectedOnly);
    if (oldConfig.fixAR != cfg->fixAR)
        emit setFixedAspectRatio(cfg->fixAR, cfg->ARmode);
    if (oldConfig.ARmode != cfg->ARmode)
        emit setFixedAspectRatio(cfg->fixAR, cfg->ARmode);
    if (oldConfig.useListingsURL != cfg->useListingsURL)
        emit useListingsURL(cfg->useListingsURL);
    if (oldConfig.listingsURL != cfg->listingsURL)
        emit getNewListingsURL(&cfg->listingsURL);
    
    // Save the new options
    cfg->save();    

} // setOptions

ChannelEditor *ViewManager::createChannelEditor( QWidget *parent )
{
    ChannelEditor *ce = new ChannelEditor( parent, "chedit", cfg->showSelectedOnly );
    
    QString cap;
    if ( parent )
        cap = i18n("Channels");
    else
        cap = kapp->makeStdCaption(i18n("Channels"));
    
    ce->setCaption( cap );
    
    connect(qtv, SIGNAL(channelChanged(Channel*)), ce, SLOT(ensureSelected(Channel*)));
    
    return ce;
}

ConfigData *ViewManager::getConfig(){
    return cfg;
}
